#!/usr/bin/perl
use RRDs;
use LWP::UserAgent;

#
#	usage		./rrd_nginx.pl rrd_database_dir png_output_dir $http_server_url $server_name
#	example		./rrd_nginx.pl /var/lib/rrd /var/www http://127.0.0.1/nginx_status localhost
#

my $rrd = "";
my $img = "";
my $URL = "";
my $ServerName = "";

# define location of rrdtool databases
if ( ! @ARGV[0] ) {
	print "\tMissing rrd_database dir!\n";
	print "\n\tUsage rrd_nginx.pl rrd_database_dir png_output_dir http_server_url server_name\n";
	print "\tExample rrd_nginx.pl /var/lib/rrd /var/www http://127.0.0.1/nginx_status localhost\n";
	exit 1;
} else {
	$rrd = @ARGV[0];
	if ( ! -d "$rrd" ) {
		print "\t Dir $rrd does not exist\n";
		exit 1;
	}
}

# define location of images
if ( ! @ARGV[1] ) {
	print "\tMissing png_output dir!\n";
	print "\n\tUsage rrd_nginx.pl rrd_database_dir png_output_dir http_server_url server_name\n";
	print "\tExample rrd_nginx.pl /var/lib/rrd /var/www http://127.0.0.1/nginx_status localhost\n";
	exit 1;
} else {
	$img = @ARGV[1];
	if ( ! -d "$img" ) {
		print "\t Dir $img does not exist\n";
		exit 1;
	}
}

# define your nginx stats URL
if ( ! @ARGV[2] ) {
	print "\tMissing http_server_url!\n";
	print "\n\tUsage rrd_nginx.pl rrd_database_dir png_output_dir http_server_url server_name\n";
	print "\tExample rrd_nginx.pl /var/lib/rrd /var/www http://127.0.0.1/nginx_status localhost\n";
	exit 1;
} else {
	$URL = @ARGV[2];
}
# define server name
if ( ! @ARGV[3] ) {
	print "\tMissing server_name!\n";
	print "\n\tUsage rrd_nginx.pl rrd_database_dir png_output_dir http_server_url server_name\n";
	print "\tExample rrd_nginx.pl /var/lib/rrd /var/www http://127.0.0.1/nginx_status localhost\n";
	exit 1;
} else {
	$ServerName = @ARGV[3];
}

my $ua = LWP::UserAgent->new(timeout => 30);
my $response = $ua->request(HTTP::Request->new('GET', $URL));

my $requests = 0;
my $total =  0;
my $reading = 0;
my $writing = 0;
my $waiting = 0;

foreach (split(/\n/, $response->content)) {
	$total = $1 if (/^Active connections:\s+(\d+)/);
	if (/^Reading:\s+(\d+).*Writing:\s+(\d+).*Waiting:\s+(\d+)/) {
		$reading = $1;
		$writing = $2;
		$waiting = $3;
	}
	$requests = $3 if (/^\s+(\d+)\s+(\d+)\s+(\d+)/);
}

#print "RQ:$requests; TT:$total; RD:$reading; WR:$writing; WA:$waiting\n";
if ( ! -e "$rrd/nginx-$ServerName.rrd" ) {
	RRDs::create "$rrd/nginx-$ServerName.rrd",
		"-s 60",
		"DS:requests:COUNTER:120:0:100000000",
		"DS:total:ABSOLUTE:120:0:60000",
		"DS:reading:ABSOLUTE:120:0:60000",
		"DS:writing:ABSOLUTE:120:0:60000",
		"DS:waiting:ABSOLUTE:120:0:60000",
		"RRA:AVERAGE:0.5:1:2880",
		"RRA:AVERAGE:0.5:30:672",
		"RRA:AVERAGE:0.5:120:732",
		"RRA:AVERAGE:0.5:720:1460";
}
# insert values into rrd database
RRDs::update "$rrd/nginx-$ServerName.rrd",
	"-t", 
	"requests:total:reading:writing:waiting",
	"N:$requests:$total:$reading:$writing:$waiting";

