#!/usr/bin/perl
use RRDs;
use LWP::UserAgent;
#
#	usage		./nginx_graph.pl rrd_database_dir png_output_dir server_name
#	example		./nginx_graph.pl /var/lib/rrd /var/www localhost
#

my $rrd = "";
my $img = "";
my $ServerName = "";

# define location of rrdtool databases
if ( ! @ARGV[0] ) {
	print "\tMissing rrd_database dir!\n";
	print "\n\tUsage nginx_graph.pl rrd_database_dir png_output_dir http_server_url server_name\n";
	print "\tExample nginx_graph.pl /var/lib/rrd /var/www localhost\n";
	exit 1;
} else {
	$rrd = @ARGV[0];
	if ( ! -d "$rrd" ) {
		print "\t Dir $rrd does not exist\n";
		exit 1;
	}
}

# define location of images
if ( ! @ARGV[1] ) {
	print "\tMissing png_output dir!\n";
	print "\n\tUsage nginx_graph.pl rrd_database_dir png_output_dir server_name\n";
	print "\tExample nginx_graph.pl /var/lib/rrd /var/www localhost\n";
	exit 1;
} else {
	$img = @ARGV[1];
	if ( ! -d "$img" ) {
		print "\t Dir $img does not exist\n";
		exit 1;
	}
}

# define server name
if ( ! @ARGV[2] ) {
	print "\tMissing server_name!\n";
	print "\n\tUsage nginx_graph.pl rrd_database_dir png_output_dir server_name\n";
	print "\tExample nginx_graph.pl /var/lib/rrd /var/www localhost\n";
	exit 1;
} else {
	$ServerName = @ARGV[2];
}

if ( ! -e "$rrd/nginx-$ServerName.rrd" ) {
	print "\n\t	File $rrd/nginx-$ServerName.rrd not found!\n";
	exit 1;
} else {
	CreateGraphs("day");
	CreateGraphs("week");
	CreateGraphs("month");
	CreateGraphs("year");
}
	

#------------------------------------------------------------------------------
sub CreateGraphs($){
	my $period = shift;
 	($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time); 
	$time_stamp = sprintf("%4d-%02d-%02d %02d:%02d:%02d",$year+1900,$mon+1,$mday,$hour,$min,$sec);
	RRDs::graph "$img/requests-$ServerName-$period.png",
		"-s -1$period",
		"-t [$time_stamp] Requests on $ServerName",
		"--lazy",
		"-h", 
		"150", 
		"-w", 
		"700",
		"-l 0",
		"-a", "PNG",
		"-v requests/sec",
		"DEF:requests=$rrd/nginx-$ServerName.rrd:requests:AVERAGE",
		"LINE2:requests#336600:Requests",
		"GPRINT:requests:MAX:  Max\\: %5.1lf %S",
		"GPRINT:requests:AVERAGE: Avg\\: %5.1lf %S",
		"GPRINT:requests:LAST: Current\\: %5.1lf %Sreq/sec",
		"HRULE:0#000000";
		
	if ($ERROR = RRDs::error) {
		print "$0: unable to generate $period graph: $ERROR\n"; 
	}

	RRDs::graph "$img/connections-$ServerName-$period.png",
		"-s -1$period",
		"-t [$time_stamp] Connection on $ServerName",
		"--lazy",
		"-h", 
		"150", 
		"-w", 
		"700",
		"-l 0",
		"-a", "PNG",
		"-v requests/sec",
		"DEF:total=$rrd/nginx-$ServerName.rrd:total:AVERAGE",
		"DEF:reading=$rrd/nginx-$ServerName.rrd:reading:AVERAGE",
		"DEF:writing=$rrd/nginx-$ServerName.rrd:writing:AVERAGE",
		"DEF:waiting=$rrd/nginx-$ServerName.rrd:waiting:AVERAGE",

		"LINE2:total#22FF22:Total",
		"GPRINT:total:LAST:   Current\\: %5.1lf %S",
		"GPRINT:total:MIN:  Min\\: %5.1lf %S",
		"GPRINT:total:AVERAGE: Avg\\: %5.1lf %S",
		"GPRINT:total:MAX:  Max\\: %5.1lf %S\\n",
		
		"LINE2:reading#0022FF:Reading",
		"GPRINT:reading:LAST: Current\\: %5.1lf %S",
		"GPRINT:reading:MIN:  Min\\: %5.1lf %S",
		"GPRINT:reading:AVERAGE: Avg\\: %5.1lf %S",
		"GPRINT:reading:MAX:  Max\\: %5.1lf %S\\n",
		
		"LINE2:writing#FF0000:Writing",
		"GPRINT:writing:LAST: Current\\: %5.1lf %S",
		"GPRINT:writing:MIN:  Min\\: %5.1lf %S",
		"GPRINT:writing:AVERAGE: Avg\\: %5.1lf %S",
		"GPRINT:writing:MAX:  Max\\: %5.1lf %S\\n",
		
		"LINE2:waiting#00AAAA:Waiting",
		"GPRINT:waiting:LAST: Current\\: %5.1lf %S",
		"GPRINT:waiting:MIN:  Min\\: %5.1lf %S",
		"GPRINT:waiting:AVERAGE: Avg\\: %5.1lf %S",
		"GPRINT:waiting:MAX:  Max\\: %5.1lf %S\\n",

		"HRULE:0#000000";

	if ($ERROR = RRDs::error) { 
		print "$0: unable to generate $period graph: $ERROR\n"; 
	}
}
